@@ -173,6 +173,45 @@ void PerlOMP_1D_Array_TO_1D_INT_ARRAY_r(SV *AVref, int numElements, int retArray
   }
 }
 
+/* 1D Array reference to 1D C string array ...
+ * Converts a Perl array of strings, e.g.,
+ *
+ *   my $Aref = [ "hello", "world", "foo", "bar" ];
+ *
+ * into a C array of strings (char*), so it can be used in OpenMP or C code.
+ */
+    
+void PerlOMP_1D_Array_TO_1D_STRING_ARRAY(SV *AVref, int numElements, char *retArray[numElements]) {
+  AV *array = (AV*)SvRV(AVref);
+  SV **element;
+  for (int i = 0; i < numElements; i++) {
+    element = av_fetch(array, i, 0);
+    if (!element || !*element || !SvOK(*element))
+      croak("Expected value at array[%d]", i);
+    
+    retArray[i] = strdup(SvPV_nolen(*element)); // Allocate and copy string
+    if (!retArray[i])
+      croak("Memory allocation failed for array[%d]", i);
+  }
+} 
+  
+/* Threaded version */
+void PerlOMP_1D_Array_TO_1D_STRING_ARRAY_r(SV *AVref, int numElements, char *retArray[numElements]) {
+  AV *array = (AV*)SvRV(AVref);
+  SV **element;
+  PerlOMP_GETENV_BASIC
+  #pragma omp parallel for
+  for (int i = 0; i < numElements; i++) {
+    element = av_fetch(array, i, 0);
+    if (!element || !*element || !SvOK(*element))
+      croak("Expected value at array[%d]", i);
+
+    retArray[i] = strdup(SvPV_nolen(*element)); // Allocate and copy string
+    if (!retArray[i])
+      croak("Memory allocation failed for array[%d]", i);
+  }
+}
+
 /* 2D AoA to 2D float C array ...
  * Convert a regular MxN Perl array of arrays (AoA) consisting of floating point values, e.g.,
  *
@@ -268,6 +307,62 @@ void PerlOMP_2D_AoA_TO_2D_INT_ARRAY_r(SV *AoA, int numRows, int rowSize, int ret
   }
 }
 
+/* 2D AoA to 2D C String array ...
+ * Convert a regular MxN Perl array of arrays (AoA) consisting of string values, e.g.,
+ *
+ *   my $AoA = [ [qw/hello world/], [qw/foo bar/], [qw/baz qux/] ];
+ *
+ * into a C array of the same dimensions (char*[][]) so it can be used with OpenMP
+ * "#pragma omp for" work-sharing construct.
+ */
+
+void PerlOMP_2D_AoA_TO_2D_STRING_ARRAY(SV *AoA, int numRows, int rowSize, char *retArray[numRows][rowSize]) {
+  SV **AVref;
+  if (!SvROK(AoA) || SvTYPE(SvRV(AoA)) != SVt_PVAV)
+    croak("Expected Arrayref");
+  
+  for (int i = 0; i < numRows; i++) {
+    AVref = av_fetch((AV*)SvRV(AoA), i, 0);
+    if (!AVref || !*AVref || !SvROK(*AVref) || SvTYPE(SvRV(*AVref)) != SVt_PVAV)
+      croak("Expected arrayref at array[%d]", i);
+
+    for (int j = 0; j < rowSize; j++) {
+      SV **element = av_fetch((AV*)SvRV(*AVref), j, 0);
+      if (!element || !*element || !SvOK(*element))
+        croak("Expected value at array[%d][%d]", i, j);
+
+      retArray[i][j] = strdup(SvPV_nolen(*element)); // Allocate and copy string
+      if (!retArray[i][j])
+        croak("Memory allocation failed for array[%d][%d]", i, j);
+    }
+  }
+}
+
+/* Threaded version using OpenMP */
+void PerlOMP_2D_AoA_TO_2D_STRING_ARRAY_r(SV *AoA, int numRows, int rowSize, char *retArray[numRows][rowSize]) {
+  SV **AVref;
+  if (!SvROK(AoA) || SvTYPE(SvRV(AoA)) != SVt_PVAV)
+    croak("Expected Arrayref");
+  
+  PerlOMP_GETENV_BASIC
+  #pragma omp parallel for private(AVref)
+  for (int i = 0; i < numRows; i++) {
+    AVref = av_fetch((AV*)SvRV(AoA), i, 0);
+    if (!AVref || !*AVref || !SvROK(*AVref) || SvTYPE(SvRV(*AVref)) != SVt_PVAV)
+      croak("Expected arrayref at array[%d]", i);
+
+    for (int j = 0; j < rowSize; j++) {
+      SV **element = av_fetch((AV*)SvRV(*AVref), j, 0);
+      if (!element || !*element || !SvOK(*element))
+        croak("Expected value at array[%d][%d]", i, j);
+
+      retArray[i][j] = strdup(SvPV_nolen(*element)); // Allocate and copy string
+      if (!retArray[i][j])
+        croak("Memory allocation failed for array[%d][%d]", i, j);
+    }
+  }
+}
+
 /* Datastructure Introspection Functions*/
 
 /**
@@ -327,6 +422,131 @@ int PerlOMP_2D_AoA_NUM_COLS(SV *AoAref) {
     return av_count(first_row);
 }
 
+/**
+  * Verification and Testing Functions
+  * ChatGPT Generated
+*/
+
+/* Helper function to check if an SV is an array reference */
+bool is_array_ref(SV *sv) {
+    return SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVAV;
+}
+
+/* Verify if a Perl variable is a valid 1D array reference */
+void PerlOMP_VERIFY_1D_Array(SV *array) {
+    if (!is_array_ref(array)) {
+        croak("Expected a 1D array reference");
+    }
+}
+
+/* Verify if a Perl variable is a valid 2D array of arrays reference */
+void PerlOMP_VERIFY_2D_AoA(SV *AoA) {
+    if (!is_array_ref(AoA)) {
+        croak("Expected a 2D array reference");
+    }
+    AV *outer = (AV *)SvRV(AoA);
+    I32 len = av_len(outer) + 1;
+    for (I32 i = 0; i < len; i++) {
+        SV **inner_ref = av_fetch(outer, i, 0);
+        if (!inner_ref || !is_array_ref(*inner_ref)) {
+            croak("Expected a 2D array with valid inner array references at index %d", i);
+        }
+    }
+}
+
+/* Helper function to verify element types */
+bool is_float(SV *sv) { return SvNOK(sv); }
+bool is_int(SV *sv) { return SvIOK(sv); }
+bool is_string(SV *sv) { return SvPOK(sv); }
+
+/* Generic function to verify a 1D array's element type */
+void verify_1D_array_type(SV *array, bool (*type_check)(SV *), const char *type_name) {
+    if (!is_array_ref(array)) {
+        croak("Expected a 1D array reference");
+    }
+    AV *av = (AV *)SvRV(array);
+    I32 len = av_len(av) + 1;
+    for (I32 i = 0; i < len; i++) {
+        SV **element = av_fetch(av, i, 0);
+        if (!element || !type_check(*element)) {
+            croak("Expected all elements to be %s at index %d", type_name, i);
+        }
+    }
+}
+
+/* Implement type-specific 1D array verifications */
+void PerlOMP_VERIFY_1D_FLOAT_ARRAY(SV *array) { verify_1D_array_type(array, is_float, "float"); }
+void PerlOMP_VERIFY_1D_INT_ARRAY(SV *array) { verify_1D_array_type(array, is_int, "integer"); }
+void PerlOMP_VERIFY_1D_DOUBLE_ARRAY(SV *array) { verify_1D_array_type(array, is_float, "double"); }
+void PerlOMP_VERIFY_1D_CHAR_ARRAY(SV *array) { verify_1D_array_type(array, is_string, "string"); }
+
+/* Check for mixed types */
+void PerlOMP_VERIFY_1D_MIXED_ARRAY(SV *array) {
+    if (!is_array_ref(array)) {
+        croak("Expected a 1D array reference");
+    }
+    AV *av = (AV *)SvRV(array);
+    I32 len = av_len(av) + 1;
+    bool found_int = false, found_float = false, found_string = false;
+    for (I32 i = 0; i < len; i++) {
+        SV **element = av_fetch(av, i, 0);
+        if (!element) continue;
+        found_int |= is_int(*element);
+        found_float |= is_float(*element);
+        found_string |= is_string(*element);
+    }
+    if (!(found_int + found_float + found_string > 1)) {
+        croak("Expected mixed types, but found only one type");
+    }
+}
+
+/* Generic function to verify a 2D array's element type */
+void verify_2D_array_type(SV *AoA, bool (*type_check)(SV *), const char *type_name) {
+    PerlOMP_VERIFY_2D_AoA(AoA);
+    AV *outer = (AV *)SvRV(AoA);
+    I32 rows = av_len(outer) + 1;
+    for (I32 i = 0; i < rows; i++) {
+        SV **inner_ref = av_fetch(outer, i, 0);
+        AV *inner = (AV *)SvRV(*inner_ref);
+        I32 cols = av_len(inner) + 1;
+        for (I32 j = 0; j < cols; j++) {
+            SV **element = av_fetch(inner, j, 0);
+            if (!element || !type_check(*element)) {
+                croak("Expected all elements to be %s at [%d][%d]", type_name, i, j);
+            }
+        }
+    }
+}
+
+/* Implement type-specific 2D array verifications */
+void PerlOMP_VERIFY_2D_FLOAT_ARRAY(SV *AoA) { verify_2D_array_type(AoA, is_float, "float"); }
+void PerlOMP_VERIFY_2D_INT_ARRAY(SV *AoA) { verify_2D_array_type(AoA, is_int, "integer"); }
+void PerlOMP_VERIFY_2D_DOUBLE_ARRAY(SV *AoA) { verify_2D_array_type(AoA, is_float, "double"); }
+void PerlOMP_VERIFY_2D_STRING_ARRAY(SV *AoA) { verify_2D_array_type(AoA, is_string, "string"); }
+
+/* Check for mixed types in a 2D array */
+void PerlOMP_VERIFY_2D_MIXED_ARRAY(SV *AoA) {
+    PerlOMP_VERIFY_2D_AoA(AoA);
+    AV *outer = (AV *)SvRV(AoA);
+    I32 rows = av_len(outer) + 1;
+    for (I32 i = 0; i < rows; i++) {
+        SV **inner_ref = av_fetch(outer, i, 0);
+        AV *inner = (AV *)SvRV(*inner_ref);
+        I32 cols = av_len(inner) + 1;
+        bool found_int = false, found_float = false, found_string = false;
+        for (I32 j = 0; j < cols; j++) {
+            SV **element = av_fetch(inner, j, 0);
+            if (!element) continue;
+            found_int |= is_int(*element);
+            found_float |= is_float(*element);
+            found_string |= is_string(*element);
+        }
+        if (!(found_int + found_float + found_string > 1)) {
+            croak("Expected mixed types in row %d, but found only one type", i);
+        }
+    }
+}
+
 /* TODO:
   * add unit tests for conversion functions
   * add some basic matrix operations (transpose for 2D, reverse for 1D)
